/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/



#ifndef __FOO_BLUETOOTH_CTRL_H__
#define __FOO_BLUETOOTH_CTRL_H__

#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"

#include "resource.h"
#include "connection.h"
#include "macros.h"
#include "bluetooth_control.h"
#include "control.h"
#include "foobar_control.h"


#define WM_BT_CMD	0x8000

/** 
 * GUIDs for the configuration settings
 */
// {B3BBE560-14AE-4f72-B46B-3736497FB0DA}
static const GUID foo_bluetooth_ctrl_preference_guid = 
{ 0xb3bbe560, 0x14ae, 0x4f72, { 0xb4, 0x6b, 0x37, 0x36, 0x49, 0x7f, 0xb0, 0xda } };
// {3A288ADA-2D70-419f-9A23-117590C800A9}
static const GUID config_comport_GUID = 
{ 0x3a288ada, 0x2d70, 0x419f, { 0x9a, 0x23, 0x11, 0x75, 0x90, 0xc8, 0x0, 0xa9 } };
// {05E9A1C5-7427-4c79-B84F-AA358AC3DB6D}
static const GUID config_musicdir_GUID = 
{ 0x5e9a1c5, 0x7427, 0x4c79, { 0xb8, 0x4f, 0xaa, 0x35, 0x8a, 0xc3, 0xdb, 0x6d } };
// {3C5362C6-5B43-4b3f-A7C0-95414A4F63C0}
static const GUID config_playlist_fmt_GUID = 
{ 0x3c5362c6, 0x5b43, 0x4b3f, { 0xa7, 0xc0, 0x95, 0x41, 0x4a, 0x4f, 0x63, 0xc0 } };
// {D785C63A-0D05-4dbd-B086-46F2D9526840}
static const GUID config_autoconnect_GUID = 
{ 0xd785c63a, 0xd05, 0x4dbd, { 0xb0, 0x86, 0x46, 0xf2, 0xd9, 0x52, 0x68, 0x40 } };
// {E95D2165-51F3-4ad0-972A-32D7E1043292}
static const GUID config_random_GUID = 
{ 0xe95d2165, 0x51f3, 0x4ad0, { 0x97, 0x2a, 0x32, 0xd7, 0xe1, 0x4, 0x32, 0x92 } };
// {9295BB38-C155-4498-8269-2A6A0EA8109B}
static const GUID config_specific_playlist_GUID = 
{ 0x9295bb38, 0xc155, 0x4498, { 0x82, 0x69, 0x2a, 0x6a, 0xe, 0xa8, 0x10, 0x9b } };
// {A571FDAC-B494-4d31-BFA7-1D944FCE3A4B}
static const GUID config_selected_playlist_GUID = 
{ 0xa571fdac, 0xb494, 0x4d31, { 0xbf, 0xa7, 0x1d, 0x94, 0x4f, 0xce, 0x3a, 0x4b } };


/**
 * config variables
 */
static cfg_string cfg_comport(config_comport_GUID, "COM4");
static cfg_string cfg_musicdir(config_musicdir_GUID, "H:\\mp3");
static cfg_string cfg_playlist_fmt(config_playlist_fmt_GUID, "$if2(%artist% - %title%, %_filename_ext%)");
static cfg_int cfg_autoconnect(config_autoconnect_GUID, 0);
static cfg_int cfg_random(config_random_GUID, 0);
static cfg_int cfg_specific_playlist(config_specific_playlist_GUID, 0);
static cfg_int cfg_selected_playlist(config_selected_playlist_GUID, 0);


/**
 * Main class
 */
class foo_bluetooth_ctrl
{
public:
	foo_bluetooth_ctrl();
	~foo_bluetooth_ctrl();

	void set_active(bool activate);
	bool is_active();
	void read_command();
	void update_config();

private:
	bluetooth_control *m_bt_control;
	connection *m_conn;
	control *m_play_control;
	HANDLE	m_io_thread;
	HWND	m_hwnd;
	bool	m_activated;

	static unsigned long WINAPI read_command_thread(LPVOID param);
	static LRESULT CALLBACK wndproc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
};


foo_bluetooth_ctrl *g_bt_ctrl = NULL;



/**
 * So that we don't have to have a lot of ugly #ifdefs all around
 * the code. Easier to read this way..
 */
void g_debug(pfc::string8 str)
{
#ifdef _DEBUG
	console::info(str);
#endif
}




/**
 * Called on foobar init/quit
 */
class initquit_bluetooth : public initquit
{
public:
	virtual void on_init()
	{ 
		g_bt_ctrl = new foo_bluetooth_ctrl();	// create new instance
		if(cfg_autoconnect != 0)
			g_bt_ctrl->set_active(true);				// activate if autoconnect is set
	}
	virtual void on_quit()
	{
		g_bt_ctrl->set_active(false);
		SAFE_DELETE(g_bt_ctrl);
	}
};




/**
 * Config menu (preferences)
 */
class config_bluetooth_ctrl : public preferences_page
{
	static BOOL CALLBACK ConfigProc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp);
	HWND create(HWND parent)
	{
		return uCreateDialog(IDD_CONFIG_BLUETOOTH_CTRL, parent, ConfigProc, 0);
	}

	virtual void reset() { 
		// TODO implement
	}

	virtual bool reset_query() {return false;}
	virtual GUID get_guid() {return foo_bluetooth_ctrl_preference_guid;}
	virtual GUID get_parent_guid() {return guid_tools;}
	virtual const char * get_name() {return "Bluetooth Remote Control";}
};




static service_factory_single_t<initquit_bluetooth> iqbt;
static service_factory_single_t<config_bluetooth_ctrl> cbtc;




DECLARE_COMPONENT_VERSION(
	"Bluetooth Remote Control",
	"1.1",
	"Copyright (c) 2004-2007 Jonatan Kronqvist <jonatan@oxygenite.org>"
);


#endif /*__FOO_BLUETOOTH_CTRL_H__*/