/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/


#include "foobar_control.h"


void foobar_control::fade_stop()
{
	// TODO: fix the actual fading :)
	stop();
}


void foobar_control::ffwd(long secs)
{
	// TODO: figure out if the param is in seconds
	// also, where the hell in the client can we seek? :)
	static_api_ptr_t<play_control> pc;
	pc->playback_seek_delta(secs);
}


void foobar_control::rwnd(long secs)
{
	// TODO: figure out if the param is in seconds
	// also, where the hell in the client can we seek? :)
	static_api_ptr_t<play_control> pc;
	pc->playback_seek_delta(-secs);
}

unsigned char foobar_control::get_volume()
{
	static_api_ptr_t<play_control> pc;
	float dBvol = pc->get_volume();
	return (unsigned char)(255 * ((100.0f-dBvol)/100.0f));	// loose some precision -> -100 .. 0 dB
}

void foobar_control::set_volume(unsigned char vol)
{
	float dBvol = (vol/256.0f * 100.0f) - 100.0f;
	// because of loss of precision:
	if(vol == 255) dBvol = 0;
	static_api_ptr_t<play_control> pc;
	pc->set_volume(dBvol);
}

void foobar_control::add_to_playlist(pfc::string8 fname)
{
	pfc::string8 url ("file://");
	url.add_string(fname);
	g_debug(url);
	static_api_ptr_t<playlist_manager> pm;
	pfc::list_single_ref_t<const char*> urls (url.get_ptr());
	pm->activeplaylist_add_locations(urls, false, NULL);
}


void foobar_control::next(bool random)
{
	static_api_ptr_t<play_control> pc;
	if(m_shuffle && random)
		pc->play_start(play_control::track_command_rand);
	else
		pc->play_start(play_control::track_command_next);
}


void foobar_control::prev()
{
	static_api_ptr_t<play_control> pc;
	pc->play_start(play_control::track_command_prev);
}


void foobar_control::pause()
{
	static_api_ptr_t<play_control> pc;
	pc->toggle_pause();
}



void foobar_control::stop()
{
	static_api_ptr_t<play_control> pc;
	pc->play_stop();
}


void foobar_control::play(pfc::string8 fname)
{
	pfc::string8 url ("file://");
	url.add_string(fname);
	g_debug(url);

	static_api_ptr_t<playlist_manager> pm;
	// add the song to the playlist and play it
	pfc::list_single_ref_t<const char*> urls (url.get_ptr());
	pm->activeplaylist_add_locations(urls, true, NULL);
	pm->activeplaylist_execute_default_action(pm->activeplaylist_get_item_count()-1);
}


void foobar_control::play_current()
{
	static_api_ptr_t<play_control> pc;
	if (pc->is_paused())
		pc->toggle_pause();
	else
		pc->play_start();
}

long foobar_control::get_playlist_len()
{
	static_api_ptr_t<playlist_manager> pm;
	return pm->activeplaylist_get_item_count();
}

std::vector<pfc::string8> *foobar_control::get_playlist()
{
	std::vector<pfc::string8> *ret = new std::vector<pfc::string8>;

	static_api_ptr_t<playlist_manager> pm;

	metadb_handle_ptr handle;
	unsigned short playlist_len = (unsigned short)get_playlist_len();
	for(int i=0; i<playlist_len; i++) {
		pm->activeplaylist_get_item_handle(handle, i);
		pfc::string8 title;
		static_api_ptr_t<titleformat_compiler> tfc;
		service_ptr_t<titleformat_object> playlist_fmt_compiled;
		tfc->compile(playlist_fmt_compiled, m_playlist_fmt);
		handle->format_title(NULL, title, playlist_fmt_compiled, NULL);
		title.add_char('\n');
		ret->push_back(title);
	}
	return ret;
}

void foobar_control::select_at(int idx)
{
	pfc::string8 str ("Selected track for play at: ");
	str << idx;
	g_debug(str);

	static_api_ptr_t<playlist_manager> pm;
	pm->activeplaylist_set_focus_item(idx);
	pm->activeplaylist_execute_default_action(idx);
}


void foobar_control::clear_playlist()
{
	static_api_ptr_t<playlist_manager> pm;
	pm->activeplaylist_clear();
}

void foobar_control::set_playback_order_by_name(pfc::string8 order)
{
	static_api_ptr_t<playlist_manager> pm;
	for(size_t i=0; i<pm->playback_order_get_count(); i++) {
		if (strncmp(pm->playback_order_get_name(i), order.get_ptr(), order.length()) == 0) {
			pm->playback_order_set_active((pm->playback_order_get_active()));
			return;
		}
	}
}

void foobar_control::set_repeat(bool repeat)
{
	if (repeat)
		this->set_playback_order_by_name("Repeat (playlist)");
	else
		this->set_playback_order_by_name("Default");
}


void foobar_control::set_shuffle(bool shuffle)
{
	if(shuffle)
		this->set_playback_order_by_name("Shuffle (tracks)");
	else
		this->set_playback_order_by_name("Default");
}


void foobar_control::seek(long pos)
{
	static_api_ptr_t<play_control> pc;
	if(pc->playback_can_seek())
		pc->playback_seek((double)pos);
}


void foobar_control::set_stop_at_eof_track(bool stop)
{
	static_api_ptr_t<play_control> pc;
	pc->set_stop_after_current(stop);
}


long foobar_control::get_bitrate()
{
	static_api_ptr_t<play_control> pc;
	file_info_i finf;
	metadb_handle_ptr h;
	pc->get_now_playing(h);
	h->metadb_lock();
	h->get_info(finf);
	long bitrate = (long)finf.info_get_bitrate();
	h->metadb_unlock();

	return bitrate;
}

long foobar_control::get_samplerate()
{
/*	static_api_ptr_t<play_control> pc;
	file_info_i finf;
	metadb_handle_ptr h;
	pc->get_now_playing(h);
	h->metadb_lock();
	h->get_info(finf);

	audio_chunk_i chunk;
	input_helper in;
	in.open(NULL, h, 0, NULL);
	in.run(&chunk);
	long samplerate = chunk.get_srate();
	in.close();

	h->metadb_unlock();
	
	return samplerate;
	*/
	// FIXME:
	return 1337;
}

long foobar_control::get_channels()
{
/*	static_api_ptr_t<play_control> pc;
	file_info_i_full finf;
	metadb_handle *h = pc->get_now_playing();
	h->handle_lock();
	h->handle_query(&finf);

	audio_chunk_i chunk;
	input_helper in;
	in.open(h);
	in.run(&chunk);
	long channels = chunk.get_channels();
	in.close();

	h->handle_unlock();
	h->handle_release();

	return channels;
^*/
	// FIXME:
	return 2;
}

const char* foobar_control::get_file_path()
{
	static_api_ptr_t<play_control> pc;
	file_info_i finf;
	metadb_handle_ptr h;
	pc->get_now_playing(h);

	return h->get_path();
}

long foobar_control::get_playback_time()
{
	static_api_ptr_t<play_control> pc;
	return (long)pc->playback_get_position();
}

long foobar_control::get_playlength()
{
	static_api_ptr_t<play_control> pc;
	return (long)pc->playback_get_length();
}

bool foobar_control::get_shuffle()
{
	static_api_ptr_t<playlist_manager> pm;
	pfc::string8 tmpstr (pm->playback_order_get_name(pm->playback_order_get_active()));
	m_shuffle = (tmpstr.find_first("Shuffle") != -1);
	return m_shuffle;
}

bool foobar_control::get_repeat()
{
	static_api_ptr_t<playlist_manager> pm;
	pfc::string8 tmpstr (pm->playback_order_get_name(pm->playback_order_get_active()));
	return (tmpstr.find_first("Repeat") != -1);
}

void foobar_control::get_title(pfc::string8 &str)
{
	static_api_ptr_t<play_control> pc;
	static_api_ptr_t<titleformat_compiler> tfc;
	service_ptr_t<titleformat_object> playlist_fmt_compiled;
	tfc->compile(playlist_fmt_compiled, m_playlist_fmt);
	pc->playback_format_title(NULL, str, playlist_fmt_compiled, NULL, playback_control::display_level_titles);
}

bool foobar_control::is_paused()
{
	static_api_ptr_t<play_control> pc;
	return pc->is_paused();
}

bool foobar_control::is_playing()
{
	static_api_ptr_t<play_control> pc;
	return pc->is_playing();
}

long foobar_control::get_cur_idx()
{
	static_api_ptr_t<playlist_manager> pm;
	t_size ix_playlist, ix_item;
	pm->get_playing_item_location(&ix_playlist, &ix_item);
	return (long)ix_item;
}


void foobar_control::set_playlist_fmt(pfc::string8 playlist_fmt)
{
	m_playlist_fmt = playlist_fmt;
}
